#import "IosFlutterAiPlugin.h"
#if __has_include(<ios_flutter_ai_plugin/ios_flutter_ai_plugin-Swift.h>)
#import <ios_flutter_ai_plugin/ios_flutter_ai_plugin-Swift.h>
#else
// Support project import fallback if the generated compatibility header
// is not copied when this plugin is created as a library.
// https://forums.swift.org/t/swift-static-libraries-dont-copy-generated-objective-c-header/19816
#import "ios_flutter_ai_plugin-Swift.h"
#endif
#import "TAIOnLineTool.h"
#import "PrivateInfo.h"

@interface IosFlutterAiPlugin()<FlutterStreamHandler>

@property (nonatomic, strong) TAIOnLineTool *aiTool;

@end

@implementation IosFlutterAiPlugin

+ (instancetype)sharedInstance {
    static IosFlutterAiPlugin *instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[self alloc] init];
    });
    return instance;
}
    
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:@"com.tlwl.ios_flutter_ai_plugin" binaryMessenger:[registrar messenger]];
    IosFlutterAiPlugin *instance = [[IosFlutterAiPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
    
    //-----------注册native向flutter发送数据通道----------
    FlutterEventChannel *recordResultChannel = [FlutterEventChannel eventChannelWithName:@"native_flutter_plugin_record_result" binaryMessenger:[registrar messenger]];
    [recordResultChannel setStreamHandler:instance];
}

-(void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    if ([call.method isEqualToString:@"initPlugin"]) {
        [self init:call.arguments result:result];
    } else if ([call.method isEqualToString:@"startRecord"]) {
        [self.aiTool startRecordParams:call.arguments result:result];
    } else {
        // 找不到方法
        result(FlutterMethodNotImplemented);
    }
}

///  初始化
- (void)init:(NSDictionary*)arguments result:(FlutterResult)result{
    [PrivateInfo shareInstance].appId = arguments[@"appId"];
    [PrivateInfo shareInstance].secretId = arguments[@"secretId"];
    [PrivateInfo shareInstance].secretKey = arguments[@"secretKey"];
    result([NSNumber numberWithBool:YES]);
}

- (FlutterError *)onListenWithArguments:(id)arguments eventSink:(FlutterEventSink)events {
    
    NSArray *argumentsList = arguments;
    for (int i = 0; i<argumentsList.count; i++) {
        TLEventStreamMode argument = [argumentsList[i] integerValue];
        switch (argument) {
            case TLEventStreamModeRecordResult:
                [IosFlutterAiPlugin sharedInstance].recordResultSink = events;
                break;
            case TLEventStreamModeRecordStart:
                [IosFlutterAiPlugin sharedInstance].startRecordSink = events;
                break;
            case TLEventStreamModeRecordStop:
                [IosFlutterAiPlugin sharedInstance].stopRecordSink = events;
                break;
            case TLEventStreamModeVolume:
                [IosFlutterAiPlugin sharedInstance].volumeSink = events;
                break;
            case TLEventStreamModeAudioPath:
                [IosFlutterAiPlugin sharedInstance].audioPathSink = events;
                break;
            default:
                break;
        }
    }
    return nil;
}

- (FlutterError *)onCancelWithArguments:(id)arguments {
    self.recordResultSink = nil;
    self.startRecordSink = nil;
    self.stopRecordSink = nil;
    self.volumeSink = nil;
    self.audioPathSink = nil;
    return nil;
}

- (TAIOnLineTool *)aiTool {
    if (!_aiTool) {
        _aiTool = [[TAIOnLineTool alloc] init];
    }
    return _aiTool;
}

@end
