//
//  TAIOnLineTool.m
//  TAIOnLineDemo
//
//  Created by 刘忠华 on 2022/6/24.
//

#import "TAIOnLineTool.h"
#import <TAISDK/TAIOralEvaluation.h>
#import "PrivateInfo.h"
#import <UIKit/UIKit.h>
#import "IosFlutterAiPlugin.h"

@interface TAIOnLineTool() <TAIOralEvaluationDelegate>

@property (strong, nonatomic) TAIOralEvaluation *oralEvaluation;
/**日志打印**/
@property (nonatomic, copy) NSString *log;
@end

@implementation TAIOnLineTool


- (void)startRecordParams:(id)params result:(FlutterResult)result {
    NSString *exampleText = params[@"exampleText"];
    TLExampleContentMode exampleMode = [params[@"exampleMode"] integerValue];
    [self startRecordForExampleText:exampleText exampleMode:exampleMode];
}

- (void)startRecordForExampleText:(NSString *)exampleText exampleMode:(TLExampleContentMode)exampleMode {
    if([self.oralEvaluation isRecording]){
        __weak typeof(self) ws = self;
        [self.oralEvaluation stopRecordAndEvaluation:^(TAIError *error) {
            [ws setResponse:[NSString stringWithFormat:@"stopRecordAndEvaluation:%@", error]];
            if ([IosFlutterAiPlugin sharedInstance].stopRecordSink) {
                NSMutableDictionary *dict = [NSMutableDictionary dictionary];
                [dict setValue:@(error.code) forKey:@"stopRecordErrCodeKey"];
                [IosFlutterAiPlugin sharedInstance].stopRecordSink(dict);
            }
            
            if ([IosFlutterAiPlugin sharedInstance].stopRecordSink) {
                NSMutableDictionary *dict = [NSMutableDictionary dictionary];
                [dict setValue:self.audioPath forKey:@"audioPathKey"];
                [IosFlutterAiPlugin sharedInstance].audioPathSink(dict);
            }
        }];
        return;
    }
   
    if([exampleText isEqualToString:@""]){
        return;
    }
    
    TAIOralEvaluationParam *param = [[TAIOralEvaluationParam alloc] init];
    param.sessionId = [[NSUUID UUID] UUIDString];
    param.appId = [PrivateInfo shareInstance].appId;
    param.soeAppId = [PrivateInfo shareInstance].soeAppId;
    param.secretId = [PrivateInfo shareInstance].secretId;
    param.secretKey = [PrivateInfo shareInstance].secretKey;
    param.token = [PrivateInfo shareInstance].token;
    param.workMode = (TAIOralEvaluationWorkMode)TAIOralEvaluationWorkMode_Once;
    param.evalMode = (TAIOralEvaluationEvalMode)exampleMode;
    param.serverType = (TAIOralEvaluationServerType)TAIOralEvaluationServerType_Chinese;
    param.hostType=(TAIOralEvaluationHostType)TAIOralEvaluationHostType_Overseas;
    param.scoreCoeff = 1;
    param.fileType = TAIOralEvaluationFileType_Mp3;
    param.textMode = (TAIOralEvaluationTextMode)TAIOralEvaluationTextMode_Noraml;
    param.refText = exampleText;
    self.audioPath = [NSString stringWithFormat:@"%@/%@.mp3", NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES)[0], param.sessionId];
    param.audioPath = self.audioPath;
    
    if(param.workMode == TAIOralEvaluationWorkMode_Stream){
        param.timeout = 5;
        param.retryTimes = 5;
    }
    else{
        param.timeout = 30;
        param.retryTimes = 0;
    }
    CGFloat fragSize = 1.0;
    if(fragSize == 0){
        return;
    }
    TAIRecorderParam *recordParam = [[TAIRecorderParam alloc] init];
    recordParam.fragEnable = (param.workMode == TAIOralEvaluationWorkMode_Stream ? YES: NO);
    recordParam.fragSize = fragSize * 1024;
    recordParam.vadEnable = YES;
    recordParam.vadInterval = 5000;
    [self.oralEvaluation setRecorderParam:recordParam];
    
    __weak typeof(self) ws = self;
    [self.oralEvaluation resetAvAudioSession:true];
    [self.oralEvaluation startRecordAndEvaluation:param callback:^(TAIError *error) {
        if ([IosFlutterAiPlugin sharedInstance].startRecordSink) {
            NSMutableDictionary *dict = [NSMutableDictionary dictionary];
            [dict setValue:@(error.code) forKey:@"startRecordErrCodeKey"];
            [IosFlutterAiPlugin sharedInstance].startRecordSink(dict);
        }
        [ws setResponse:[NSString stringWithFormat:@"startRecordAndEvaluation:%@", error]];
    }];
}

- (void)setResponse:(NSString *)string
{
    NSDateFormatter *format = [[NSDateFormatter alloc] init];
    [format setDateFormat:@"yyyy-MM-dd HH:mm:ss.SSS"];
//    NSString *desc = [NSString stringWithCString:[string cStringUsingEncoding:NSUTF8StringEncoding] encoding:NSNonLossyASCIIStringEncoding];
    self.log = [NSString stringWithFormat:@"%@\n%@ %@", self.log, [format stringFromDate:[NSDate date]], string];
//    NSLog(@"Response--%@",self.log);
}

#pragma mark - oral evaluation delegate
- (void)oralEvaluation:(TAIOralEvaluation *)oralEvaluation onEvaluateData:(TAIOralEvaluationData *)data result:(TAIOralEvaluationRet *)result error:(TAIError *)error
{
    NSString *resultStr = [NSString stringWithFormat:@"oralEvaluation:seq:%ld, end:%ld, error:%@, ret:%@", (long)data.seqId, (long)data.bEnd, error, result];
    //native向flutter传输数据流
    if ([IosFlutterAiPlugin sharedInstance].recordResultSink) {
        NSMutableDictionary *dict = [NSMutableDictionary dictionary];
        [dict setValue:resultStr forKey:@"recordResultKey"];
        [IosFlutterAiPlugin sharedInstance].recordResultSink(dict);
    }
}

//检测到静音
- (void)onEndOfSpeechInOralEvaluation:(TAIOralEvaluation *)oralEvaluation
{
    //这里可以根据业务逻辑处理，如停止录音或提示用户
    [self setResponse:@"onEndOfSpeech"];
    [self startRecordForExampleText:nil exampleMode:0];
}

#pragma mark - 监听音量发生变化
- (void)oralEvaluation:(TAIOralEvaluation *)oralEvaluation onVolumeChanged:(NSInteger)volume
{
    //回调录音分贝大小[0-120]
    if ([IosFlutterAiPlugin sharedInstance].volumeSink) {
        NSMutableDictionary *dict = [NSMutableDictionary dictionary];
        [dict setValue:@(volume) forKey:@"volumeKey"];
        [IosFlutterAiPlugin sharedInstance].volumeSink(dict);
    }
}

- (TAIOralEvaluation *)oralEvaluation
{
    if(!_oralEvaluation){
        _oralEvaluation = [[TAIOralEvaluation alloc] init];
        _oralEvaluation.delegate = self;
    }
    return _oralEvaluation;
}

@end
